@@ -38,13 +38,17 @@
 #include <string.h>
 #include <uuid/uuid.h>
 
-#include <common/na-gconf-keys.h>
+#include <runtime/na-gconf-provider-keys.h>
+#include <runtime/na-gconf-utils.h>
+
+#include <common/na-iprefs.h>
+#include <common/na-object-api.h>
 #include <common/na-utils.h>
 #include <common/na-xml-names.h>
 
 #include "nact-application.h"
 #include "nact-main-window.h"
-#include "nact-assistant.h"
+#include "nact-assistant-import-ask.h"
 #include "nact-xml-reader.h"
 
 /* private class data
@@ -60,14 +64,17 @@ struct NactXMLReaderClassPrivate {
  */
 struct NactXMLReaderPrivate {
 	gboolean         dispose_has_run;
-	NactWindow      *window;
-	NAAction        *action;			/* the action that we will return, or NULL */
+	BaseWindow      *window;
+	gint             import_mode;
+	const gchar     *uri;
+	GList           *auxiliaries;
+	NAObjectAction  *action;			/* the action that we will return, or NULL */
 	GSList          *messages;
 	gboolean         uuid_set;			/* set at first uuid, then checked against */
 
 	/* following values are reset at each schema/entry node
 	 */
-	NAActionProfile *profile;			/* profile */
+	NAObjectProfile *profile;			/* profile */
 	gboolean         locale_waited;		/* does this require a locale ? */
 	gboolean         profile_waited;	/* does this entry apply to a profile ? */
 	gboolean         list_waited;
@@ -87,10 +94,17 @@ typedef struct {
 
 static GConfReaderStruct reader_str[] = {
 	{ ACTION_VERSION_ENTRY      , FALSE, FALSE, FALSE, FALSE },
-	{ ACTION_LABEL_ENTRY        , FALSE,  TRUE, FALSE, FALSE },
-	{ ACTION_TOOLTIP_ENTRY      , FALSE,  TRUE, FALSE, FALSE },
-	{ ACTION_ICON_ENTRY         , FALSE, FALSE, FALSE, FALSE },
-	{ ACTION_ENABLED_ENTRY      , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_LABEL_ENTRY   , FALSE,  TRUE, FALSE, FALSE },
+	{ OBJECT_ITEM_TOOLTIP_ENTRY , FALSE,  TRUE, FALSE, FALSE },
+	{ OBJECT_ITEM_ICON_ENTRY    , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_ENABLED_ENTRY , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_LIST_ENTRY    , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_TYPE_ENTRY    , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_TARGET_SELECTION_ENTRY  , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_TARGET_BACKGROUND_ENTRY , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_TARGET_TOOLBAR_ENTRY    , FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_TOOLBAR_SAME_LABEL_ENTRY, FALSE, FALSE, FALSE, FALSE },
+	{ OBJECT_ITEM_TOOLBAR_LABEL_ENTRY     ,  TRUE, FALSE, FALSE, FALSE },
 	{ ACTION_PROFILE_LABEL_ENTRY, FALSE,  TRUE,  TRUE, FALSE },
 	{ ACTION_PATH_ENTRY         , FALSE, FALSE,  TRUE, FALSE },
 	{ ACTION_PARAMETERS_ENTRY   , FALSE, FALSE,  TRUE, FALSE },
@@ -101,6 +115,7 @@ static GConfReaderStruct reader_str[] = {
 	{ ACTION_MULTIPLE_ENTRY     , FALSE, FALSE,  TRUE, FALSE },
 	{ ACTION_MIMETYPES_ENTRY    , FALSE, FALSE,  TRUE,  TRUE },
 	{ ACTION_SCHEMES_ENTRY      , FALSE, FALSE,  TRUE,  TRUE },
+	{ ACTION_FOLDERS_ENTRY      , FALSE, FALSE,  TRUE,  TRUE },
 	{                       NULL, FALSE, FALSE, FALSE, FALSE },
 };
 
@@ -116,46 +131,48 @@ static GConfReaderStruct reader_str[] = {
 #define ERR_INVALID_UUID			_( "Invalid UUID: waited for %s, found %s at line %d." )
 #define ERR_INVALID_KEY_PREFIX		_( "Invalid content: waited for %s prefix, found %s at line %d." )
 #define ERR_NOT_AN_UUID				_( "Invalid UUID %s found at line %d." )
-#define ERR_UUID_ALREADY_EXISTS		_( "Already existing action (UUID: %s) at line %d." )
+#define ERR_UUID_ALREADY_EXISTS		_( "Already existing action (UUID: %s)." )
 #define ERR_VALUE_ALREADY_SET		_( "Value '%s' already set: new value ignored at line %d." )
 #define ERR_UUID_NOT_FOUND			_( "UUID not found." )
 #define ERR_ACTION_LABEL_NOT_FOUND	_( "Action label not found." )
 
 static GObjectClass *st_parent_class = NULL;
 
-static GType            register_type( void );
-static void             class_init( NactXMLReaderClass *klass );
-static void             instance_init( GTypeInstance *instance, gpointer klass );
-static void             instance_dispose( GObject *object );
-static void             instance_finalize( GObject *object );
+static GType          register_type( void );
+static void           class_init( NactXMLReaderClass *klass );
+static void           instance_init( GTypeInstance *instance, gpointer klass );
+static void           instance_dispose( GObject *object );
+static void           instance_finalize( GObject *object );
 
 static NactXMLReader *gconf_reader_new( void );
 
-static void             gconf_reader_parse_schema_root( NactXMLReader *reader, xmlNode *root );
-static void             gconf_reader_parse_schemalist( NactXMLReader *reader, xmlNode *schemalist );
-static gboolean         gconf_reader_parse_schema( NactXMLReader *reader, xmlNode *schema );
-static gboolean         gconf_reader_parse_applyto( NactXMLReader *reader, xmlNode *node );
-static gboolean         gconf_reader_check_for_entry( NactXMLReader *reader, xmlNode *node, const char *entry );
-static gboolean         gconf_reader_parse_locale( NactXMLReader *reader, xmlNode *node );
-static void             gconf_reader_parse_default( NactXMLReader *reader, xmlNode *node );
-static gchar           *get_profile_name_from_schema_key( const gchar *key, const gchar *uuid );
+static void           gconf_reader_parse_schema_root( NactXMLReader *reader, xmlNode *root );
+static void           gconf_reader_parse_schemalist( NactXMLReader *reader, xmlNode *schemalist );
+static gboolean       gconf_reader_parse_schema( NactXMLReader *reader, xmlNode *schema );
+static gboolean       gconf_reader_parse_applyto( NactXMLReader *reader, xmlNode *node );
+static gboolean       gconf_reader_check_for_entry( NactXMLReader *reader, xmlNode *node, const char *entry );
+static gboolean       gconf_reader_parse_locale( NactXMLReader *reader, xmlNode *node );
+static void           gconf_reader_parse_default( NactXMLReader *reader, xmlNode *node );
+static gchar         *get_profile_name_from_schema_key( const gchar *key, const gchar *uuid );
 
-static void             gconf_reader_parse_dump_root( NactXMLReader *reader, xmlNode *root );
-static void             gconf_reader_parse_entrylist( NactXMLReader *reader, xmlNode *entrylist );
-static gboolean         gconf_reader_parse_entry( NactXMLReader *reader, xmlNode *entry );
-static gboolean         gconf_reader_parse_dump_key( NactXMLReader *reader, xmlNode *key );
-static void             gconf_reader_parse_dump_value( NactXMLReader *reader, xmlNode *key );
-static void             gconf_reader_parse_dump_value_list( NactXMLReader *reader, xmlNode *key );
-static gchar           *get_profile_name_from_dump_key( const gchar *key );
+static void           gconf_reader_parse_dump_root( NactXMLReader *reader, xmlNode *root );
+static void           gconf_reader_parse_entrylist( NactXMLReader *reader, xmlNode *entrylist );
+static gboolean       gconf_reader_parse_entry( NactXMLReader *reader, xmlNode *entry );
+static gboolean       gconf_reader_parse_dump_key( NactXMLReader *reader, xmlNode *key );
+static void           gconf_reader_parse_dump_value( NactXMLReader *reader, xmlNode *key );
+static void           gconf_reader_parse_dump_value_list( NactXMLReader *reader, xmlNode *key );
+static gchar         *get_profile_name_from_dump_key( const gchar *key );
 
-static void             apply_values( NactXMLReader *reader );
-static void             add_message( NactXMLReader *reader, const gchar *format, ... );
-static int              strxcmp( const xmlChar *a, const char *b );
-static gchar           *get_uuid_from_key( NactXMLReader *reader, const gchar *key, guint line );
-static gboolean         is_uuid_valid( const gchar *uuid );
-static gchar           *get_entry_from_key( const gchar *key );
-static void             free_reader_values( NactXMLReader *reader );
-static gboolean         action_exists( NactXMLReader *reader, const gchar *uuid );
+static void           apply_values( NactXMLReader *reader );
+static void           add_message( NactXMLReader *reader, const gchar *format, ... );
+static int            strxcmp( const xmlChar *a, const char *b );
+static gchar         *get_uuid_from_key( NactXMLReader *reader, const gchar *key, guint line );
+static gboolean       is_uuid_valid( const gchar *uuid );
+static gchar         *get_entry_from_key( const gchar *key );
+static void           free_reader_values( NactXMLReader *reader );
+static gboolean       manage_import_mode( NactXMLReader *reader );
+static NAObjectItem  *search_in_auxiliaries( NactXMLReader *reader, const gchar *uuid );
+static void           relabel( NactXMLReader *reader );
 
 GType
 nact_xml_reader_get_type( void )
@@ -218,7 +235,7 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	NactXMLReader *self;
 
 	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
-	g_assert( NACT_IS_XML_READER( instance ));
+	g_return_if_fail( NACT_IS_XML_READER( instance ));
 	self = NACT_XML_READER( instance );
 
 	self->private = g_new0( NactXMLReaderPrivate, 1 );
@@ -239,8 +256,8 @@ instance_dispose( GObject *object )
 	static const gchar *thisfn = "nact_xml_reader_instance_dispose";
 	NactXMLReader *self;
 
-	g_assert( NACT_IS_XML_READER( object ));
 	g_debug( "%s: object=%p", thisfn, ( void * ) object );
+	g_return_if_fail( NACT_IS_XML_READER( object ));
 	self = NACT_XML_READER( object );
 
 	if( !self->private->dispose_has_run ){
@@ -248,12 +265,14 @@ instance_dispose( GObject *object )
 		self->private->dispose_has_run = TRUE;
 
 		if( self->private->action ){
-			g_assert( NA_IS_ACTION( self->private->action ));
+			g_return_if_fail( NA_IS_OBJECT_ACTION( self->private->action ));
 			g_object_unref( self->private->action );
 		}
 
 		/* chain up to the parent class */
-		G_OBJECT_CLASS( st_parent_class )->dispose( object );
+		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
+			G_OBJECT_CLASS( st_parent_class )->dispose( object );
+		}
 	}
 }
 
@@ -263,8 +282,8 @@ instance_finalize( GObject *object )
 	static const gchar *thisfn = "nact_xml_reader_instance_finalize";
 	NactXMLReader *self;
 
-	g_assert( NACT_IS_XML_READER( object ));
 	g_debug( "%s: object=%p", thisfn, ( void * ) object );
+	g_return_if_fail( NACT_IS_XML_READER( object ));
 	self = NACT_XML_READER( object );
 
 	na_utils_free_string_list( self->private->messages );
@@ -273,7 +292,7 @@ instance_finalize( GObject *object )
 	g_free( self->private );
 
 	/* chain call to parent class */
-	if( st_parent_class->finalize ){
+	if( G_OBJECT_CLASS( st_parent_class )->finalize ){
 		G_OBJECT_CLASS( st_parent_class )->finalize( object );
 	}
 }
@@ -285,22 +304,37 @@ gconf_reader_new( void )
 }
 
 /**
+ * nact_xml_reader_import:
+ * @window: the #NactAssistantImport instance.
+ * @items: an auxiliary list of NAObjectItems in which the existancy of
+ *  the imported action could be checked ; this typically correspond to
+ *  actions which were previously imported in the assistant
+ * @uri: the uri of the file to import.
+ * @import_mode: the import mode.
+ * @msg: a list of error messages which may be set by this function.
+ *
  * Import the specified file as an NAAction XML description.
+ *
+ * Returns: the imported action, or NULL.
  */
-NAAction *
-nact_xml_reader_import( NactWindow *window, const gchar *uri, GSList **msg )
+NAObjectAction *
+nact_xml_reader_import( BaseWindow *window, GList *items, const gchar *uri, gint import_mode, GSList **msg )
 {
 	static const gchar *thisfn = "nact_xml_reader_import";
-	NAAction *action = NULL;
+	NAObjectAction *action = NULL;
 	NactXMLReader *reader;
 	xmlDoc *doc;
 
 	g_debug( "%s: window=%p, uri=%s, msg=%p", thisfn, ( void * ) window, uri, ( void * ) msg );
+	g_return_val_if_fail( BASE_IS_WINDOW( window ), NULL );
 
 	reader = gconf_reader_new();
 	reader->private->window = window;
+	reader->private->import_mode = import_mode;
+	reader->private->uri = uri;
+	reader->private->auxiliaries = items;
 
-	g_assert( NACT_IS_ASSISTANT( window ));
+	g_return_val_if_fail( BASE_IS_WINDOW( window ), NULL );
 
 	doc = xmlParseFile( uri );
 
@@ -331,12 +365,15 @@ nact_xml_reader_import( NactWindow *window, const gchar *uri, GSList **msg )
 	}
 
 	xmlCleanupParser();
-	*msg = na_utils_duplicate_string_list( reader->private->messages );
 
 	if( reader->private->action ){
-		g_assert( NA_IS_ACTION( reader->private->action ));
-		action = g_object_ref( reader->private->action );
+		g_assert( NA_IS_OBJECT_ACTION( reader->private->action ));
+		if( manage_import_mode( reader )){
+			action = g_object_ref( reader->private->action );
+		}
 	}
+
+	*msg = na_utils_duplicate_string_list( reader->private->messages );
 	g_object_unref( reader );
 
 	return( action );
@@ -388,7 +425,7 @@ gconf_reader_parse_schemalist( NactXMLReader *reader, xmlNode *schema )
 
 	g_debug( "%s: reader=%p, schema=%p", thisfn, ( void * ) reader, ( void * ) schema );
 
-	reader->private->action = na_action_new();
+	reader->private->action = na_object_action_new();
 	reader->private->uuid_set = FALSE;
 
 	for( iter = schema->children ; iter ; iter = iter->next ){
@@ -415,7 +452,7 @@ gconf_reader_parse_schemalist( NactXMLReader *reader, xmlNode *schema )
 	}
 
 	if( ok ){
-		gchar *label = na_action_get_label( reader->private->action );
+		gchar *label = na_object_get_label( reader->private->action );
 		if( !label || !g_utf8_strlen( label, -1 )){
 			add_message( reader, ERR_ACTION_LABEL_NOT_FOUND );
 		}
@@ -609,18 +646,11 @@ gconf_reader_parse_applyto( NactXMLReader *reader, xmlNode *node )
 
 	if( ret ){
 		if( !reader->private->uuid_set ){
-
-			if( action_exists( reader, uuid )){
-				add_message( reader, ERR_UUID_ALREADY_EXISTS, uuid );
-				ret = FALSE;
-
-			} else {
-				na_action_set_uuid( reader->private->action, uuid );
-				reader->private->uuid_set = TRUE;
-			}
+			na_object_set_id( reader->private->action, uuid );
+			reader->private->uuid_set = TRUE;
 
 		} else {
-			gchar *ref = na_action_get_uuid( reader->private->action );
+			gchar *ref = na_object_get_id( reader->private->action );
 			if( g_ascii_strcasecmp(( const gchar * ) uuid, ( const gchar * ) ref )){
 				add_message( reader, ERR_INVALID_UUID, ref, uuid, node->line );
 				ret = FALSE;
@@ -633,12 +663,12 @@ gconf_reader_parse_applyto( NactXMLReader *reader, xmlNode *node )
 		profile = get_profile_name_from_schema_key(( const gchar * ) text, uuid );
 
 		if( profile ){
-			reader->private->profile = NA_ACTION_PROFILE( na_action_get_profile( reader->private->action, profile ));
+			reader->private->profile = NA_OBJECT_PROFILE( na_object_get_item( reader->private->action, profile ));
 
 			if( !reader->private->profile ){
-				reader->private->profile = na_action_profile_new();
-				na_action_profile_set_name( reader->private->profile, profile );
-				na_action_attach_profile( reader->private->action, reader->private->profile );
+				reader->private->profile = na_object_profile_new();
+				na_object_set_id( reader->private->profile, profile );
+				na_object_action_attach_profile( reader->private->action, reader->private->profile );
 			}
 		}
 
@@ -775,7 +805,7 @@ gconf_reader_parse_default( NactXMLReader *reader, xmlNode *node )
 static gchar *
 get_profile_name_from_schema_key( const gchar *key, const gchar *uuid )
 {
-	gchar *prefix = g_strdup_printf( "%s/%s/%s", NA_GCONF_CONFIG_PATH, uuid, ACTION_PROFILE_PREFIX );
+	gchar *prefix = g_strdup_printf( "%s/%s/%s", NA_GCONF_CONFIG_PATH, uuid, OBJECT_PROFILE_PREFIX );
 	gchar *profile_name = NULL;
 	gchar *pos;
 
@@ -832,28 +862,26 @@ gconf_reader_parse_entrylist( NactXMLReader *reader, xmlNode *entrylist )
 	static const gchar *thisfn = "gconf_reader_parse_entrylist";
 	xmlChar *path;
 	gchar *uuid, *label;
+	gboolean ok;
 
 	g_debug( "%s: reader=%p, entrylist=%p", thisfn, ( void * ) reader, ( void * ) entrylist );
 
-	reader->private->action = na_action_new();
+	reader->private->action = na_object_action_new();
 	path = xmlGetProp( entrylist, ( const xmlChar * ) NACT_GCONF_DUMP_ENTRYLIST_BASE );
-	uuid = na_utils_path_to_key(( const gchar * ) path );
+	uuid = na_gconf_utils_path_to_key(( const gchar * ) path );
 	/*g_debug( "%s: uuid=%s", thisfn, uuid );*/
 
 	if( is_uuid_valid( uuid )){
-		if( action_exists( reader, uuid )){
-			add_message( reader, ERR_UUID_ALREADY_EXISTS, uuid, entrylist->line );
+		na_object_set_id( reader->private->action, uuid );
+		reader->private->uuid_set = TRUE;
 
-		} else {
-			na_action_set_uuid( reader->private->action, uuid );
-			reader->private->uuid_set = TRUE;
-		}
 	} else {
 		add_message( reader, ERR_NOT_AN_UUID, uuid, entrylist->line );
 	}
 
 	g_free( uuid );
 	xmlFree( path );
+	ok = TRUE;
 
 	if( reader->private->uuid_set ){
 		xmlNode *iter;
@@ -875,9 +903,10 @@ gconf_reader_parse_entrylist( NactXMLReader *reader, xmlNode *entrylist )
 			}
 		}
 
-		label = na_action_get_label( reader->private->action );
+		label = na_object_get_label( reader->private->action );
 		if( !label || !g_utf8_strlen( label, -1 )){
 			add_message( reader, ERR_ACTION_LABEL_NOT_FOUND );
+			ok = FALSE;
 		}
 		g_free( label );
 
@@ -886,6 +915,7 @@ gconf_reader_parse_entrylist( NactXMLReader *reader, xmlNode *entrylist )
 		reader->private->action = NULL;
 	}
 }
+
 static gboolean
 gconf_reader_parse_entry( NactXMLReader *reader, xmlNode *entry )
 {
@@ -999,12 +1029,12 @@ gconf_reader_parse_dump_key( NactXMLReader *reader, xmlNode *node )
 		profile = get_profile_name_from_dump_key(( const gchar * ) text );
 
 		if( profile ){
-			reader->private->profile = na_action_get_profile( reader->private->action, profile );
+			reader->private->profile = NA_OBJECT_PROFILE( na_object_get_item( reader->private->action, profile ));
 
 			if( !reader->private->profile ){
-				reader->private->profile = na_action_profile_new();
-				na_action_profile_set_name( reader->private->profile, profile );
-				na_action_attach_profile( reader->private->action, reader->private->profile );
+				reader->private->profile = na_object_profile_new();
+				na_object_set_id( reader->private->profile, profile );
+				na_object_action_attach_profile( reader->private->action, reader->private->profile );
 			}
 		}
 
@@ -1092,59 +1122,86 @@ get_profile_name_from_dump_key( const gchar *key )
 static void
 apply_values( NactXMLReader *reader )
 {
-	static const gchar *thisfn = "gconf_reader_apply_values";
+	static const gchar *thisfn = "nact_xml_reader_apply_values";
 
 	g_debug( "%s: reader=%p, entry=%s, value=%s",
 			thisfn, ( void * ) reader, reader->private->entry, reader->private->value );
 
 	if( reader->private->entry && strlen( reader->private->entry )){
 		if( !strcmp( reader->private->entry, ACTION_VERSION_ENTRY )){
-			na_action_set_version( reader->private->action, reader->private->value );
+			na_object_action_set_version( reader->private->action, reader->private->value );
 
-		} else if( !strcmp( reader->private->entry, ACTION_LABEL_ENTRY )){
-			na_action_set_label( reader->private->action, reader->private->value );
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_LABEL_ENTRY )){
+			na_object_set_label( reader->private->action, reader->private->value );
 
-		} else if( !strcmp( reader->private->entry, ACTION_TOOLTIP_ENTRY )){
-			na_action_set_tooltip( reader->private->action, reader->private->value );
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TOOLTIP_ENTRY )){
+			na_object_set_tooltip( reader->private->action, reader->private->value );
 
-		} else if( !strcmp( reader->private->entry, ACTION_ICON_ENTRY )){
-			na_action_set_icon( reader->private->action, reader->private->value );
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_ICON_ENTRY )){
+			na_object_set_icon( reader->private->action, reader->private->value );
 
-		} else if( !strcmp( reader->private->entry, ACTION_ENABLED_ENTRY )){
-			na_action_set_enabled( reader->private->action, na_utils_schema_to_boolean( reader->private->value, TRUE ));
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_ENABLED_ENTRY )){
+			na_object_set_enabled( NA_OBJECT_ITEM( reader->private->action ), na_utils_schema_to_boolean( reader->private->value, TRUE ));
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_LIST_ENTRY )){
+			na_object_item_set_items_string_list( NA_OBJECT_ITEM( reader->private->action ), reader->private->list_value );
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TYPE_ENTRY )){
+			/* type (Action, Menu) is not set here, because this must has been
+			 * decided at object construction (it is so too late !)
+			 */
+			;
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TARGET_SELECTION_ENTRY )){
+			na_object_action_set_target_selection( reader->private->action, na_utils_schema_to_boolean( reader->private->value, TRUE ));
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TARGET_BACKGROUND_ENTRY )){
+			na_object_action_set_target_background( reader->private->action, na_utils_schema_to_boolean( reader->private->value, FALSE ));
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TARGET_TOOLBAR_ENTRY )){
+			na_object_action_set_target_toolbar( reader->private->action, na_utils_schema_to_boolean( reader->private->value, FALSE ));
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TOOLBAR_SAME_LABEL_ENTRY )){
+			na_object_action_toolbar_set_same_label( reader->private->action, na_utils_schema_to_boolean( reader->private->value, TRUE ));
+
+		} else if( !strcmp( reader->private->entry, OBJECT_ITEM_TOOLBAR_LABEL_ENTRY )){
+			na_object_action_toolbar_set_label( reader->private->action, reader->private->value );
 
 		} else if( !strcmp( reader->private->entry, ACTION_PROFILE_LABEL_ENTRY )){
-			na_action_profile_set_label( reader->private->profile, reader->private->value );
+			na_object_set_label( reader->private->profile, reader->private->value );
 
 		} else if( !strcmp( reader->private->entry, ACTION_PATH_ENTRY )){
-			na_action_profile_set_path( reader->private->profile, reader->private->value );
+			na_object_profile_set_path( reader->private->profile, reader->private->value );
 
 		} else if( !strcmp( reader->private->entry, ACTION_PARAMETERS_ENTRY )){
-			na_action_profile_set_parameters( reader->private->profile, reader->private->value );
+			na_object_profile_set_parameters( reader->private->profile, reader->private->value );
 
 		} else if( !strcmp( reader->private->entry, ACTION_BASENAMES_ENTRY )){
-			na_action_profile_set_basenames( reader->private->profile, reader->private->list_value );
+			na_object_profile_set_basenames( reader->private->profile, reader->private->list_value );
 
 		} else if( !strcmp( reader->private->entry, ACTION_MATCHCASE_ENTRY )){
-			na_action_profile_set_matchcase( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, TRUE ));
+			na_object_profile_set_matchcase( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, TRUE ));
 
 		} else if( !strcmp( reader->private->entry, ACTION_ISFILE_ENTRY )){
-			na_action_profile_set_isfile( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, TRUE ));
+			na_object_profile_set_isfile( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, TRUE ));
 
 		} else if( !strcmp( reader->private->entry, ACTION_ISDIR_ENTRY )){
-			na_action_profile_set_isdir( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, FALSE ));
+			na_object_profile_set_isdir( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, FALSE ));
 
 		} else if( !strcmp( reader->private->entry, ACTION_MULTIPLE_ENTRY )){
-			na_action_profile_set_multiple( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, FALSE ));
+			na_object_profile_set_multiple( reader->private->profile, na_utils_schema_to_boolean( reader->private->value, FALSE ));
 
 		} else if( !strcmp( reader->private->entry, ACTION_MIMETYPES_ENTRY )){
-			na_action_profile_set_mimetypes( reader->private->profile, reader->private->list_value );
+			na_object_profile_set_mimetypes( reader->private->profile, reader->private->list_value );
 
 		} else if( !strcmp( reader->private->entry, ACTION_SCHEMES_ENTRY )){
-			na_action_profile_set_schemes( reader->private->profile, reader->private->list_value );
+			na_object_profile_set_schemes( reader->private->profile, reader->private->list_value );
+
+		} else if( !strcmp( reader->private->entry, ACTION_FOLDERS_ENTRY )){
+			na_object_profile_set_folders( reader->private->profile, reader->private->list_value );
 
 		} else {
-			g_assert_not_reached();
+			g_warning( "%s: unprocessed entry %s", thisfn, reader->private->entry );
 		}
 	}
 }
@@ -1241,10 +1298,105 @@ free_reader_values( NactXMLReader *reader )
 	}
 }
 
+/*
+ * returns TRUE if we can safely insert the action
+ * - the uuid doesn't already exist
+ * - the uuid already exist, but import mode is renumber
+ * - the uuid already exists, but import mode is override
+ */
 static gboolean
-action_exists( NactXMLReader *reader, const gchar *uuid )
+manage_import_mode( NactXMLReader *reader )
 {
-	BaseApplication *appli = base_window_get_application( BASE_WINDOW( reader->private->window ));
-	NactMainWindow *main_window = NACT_MAIN_WINDOW( base_application_get_main_window( appli ));
-	return( nact_main_window_action_exists( main_window, uuid ));
+	NAObjectItem *exists;
+	gchar *uuid;
+	BaseApplication *appli;
+	NactMainWindow *main_window;
+	gboolean ret;
+	gint mode;
+
+	appli = base_window_get_application( BASE_WINDOW( reader->private->window ));
+	main_window = NACT_MAIN_WINDOW( base_application_get_main_window( appli ));
+
+	uuid = na_object_get_id( reader->private->action );
+	exists = nact_main_window_get_item( main_window, uuid );
+	if( !exists ){
+		exists = search_in_auxiliaries( reader, uuid );
+	}
+
+	if( !exists ){
+		g_free( uuid );
+		return( TRUE );
+	}
+
+	if( reader->private->import_mode == IPREFS_IMPORT_ASK ){
+		mode = nact_assistant_import_ask_user( main_window, reader->private->uri, reader->private->action, exists );
+	} else {
+		mode = reader->private->import_mode;
+	}
+
+	switch( mode ){
+		case IPREFS_IMPORT_RENUMBER:
+			na_object_set_new_id( reader->private->action, NULL );
+			relabel( reader );
+			if( reader->private->import_mode == IPREFS_IMPORT_ASK ){
+				add_message( reader, "%s", _( "Action was renumbered due to user request." ));
+			}
+			ret = TRUE;
+			break;
+
+		case IPREFS_IMPORT_OVERRIDE:
+			if( reader->private->import_mode == IPREFS_IMPORT_ASK ){
+				add_message( reader, "%s", _( "Existing action was overriden due to user request." ));
+			}
+			ret = TRUE;
+			break;
+
+		case IPREFS_IMPORT_NO_IMPORT:
+		default:
+			add_message( reader, ERR_UUID_ALREADY_EXISTS, uuid );
+			if( reader->private->import_mode == IPREFS_IMPORT_ASK ){
+				add_message( reader, "%s", _( "Import was canceled due to user request." ));
+			}
+			ret = FALSE;
+	}
+
+	g_free( uuid );
+	return( ret );
+}
+
+static NAObjectItem *
+search_in_auxiliaries( NactXMLReader *reader, const gchar *uuid )
+{
+	NAObjectItem *action;
+	gchar *aux_uuid;
+	GList *it;
+
+	action = NULL;
+	for( it = reader->private->auxiliaries ; it && !action ; it = it->next ){
+		aux_uuid = na_object_get_id( it->data );
+		if( !strcmp( aux_uuid, uuid )){
+			action = NA_OBJECT_ITEM( it->data );
+		}
+		g_free( aux_uuid );
+	}
+	return( action );
+}
+
+/*
+ * set a new label because the action has been renumbered
+ */
+static void
+relabel( NactXMLReader *reader )
+{
+	gchar *label, *tmp;
+
+	label = na_object_get_label( reader->private->action );
+
+	/* i18n: the action has been renumbered during import operation */
+	tmp = g_strdup_printf( "%s %s", label, _( "(renumbered)" ));
+
+	na_object_set_label( reader->private->action, tmp );
+
+	g_free( tmp );
+	g_free( label );
 }
